import java.util.Set;
import java.util.TreeSet;

public class AdjacencyMatrixGraph implements Graph {
    static final byte FREE=0;
    static final byte USED=1;

    // Sei G = (V,E) der repraesentierte Graph
    byte[] status; // v in V genau dann wenn status[v] == USED
    byte[][] matrix; // (v,w) in E genau dann wenn matrix[v][w] == 1
                     // weiters gilt: wenn matrix[v][w] == 1 dann ist v,w in V
    int cnt;

    public AdjacencyMatrixGraph(int size) {
	matrix = new byte[size][size];
	status = new byte[size];
	cnt = 0;
    }


    public int size() {
	return cnt;
    }

    public boolean containsNode(int node) {
	return 0 <= node && node < status.length && status[node]==USED;
    }

    public boolean containsEdge(int source, int target) {
	return matrix[source][target] == 1;
    }

    public int freshNode() {
	int node = -1;
	int size = this.status.length;

	// Suche nach dem ersten freien Platz
	for (int i = 0; i < size; i++) {
	    if (status[i] == FREE) {
		node = i;
		break;
	    }
	}

	// Wenn kein freier Platz, Dimension verdoppeln
	if (node < 0) {
	    int newsize = size * 2;
	    byte[] newstatus = new byte[newsize];
	    byte[][] newmatrix = new byte[newsize][newsize];

	    System.arraycopy(status, 0, newstatus, 0, size);
	    for(int i = 0; i < size; i++) {
		System.arraycopy(matrix[i], 0, newmatrix, 0, size);
	    }
	    matrix = newmatrix; 
	    status = newstatus;
	}

	cnt++;
	status[node] = USED;
	return node;
    }

    public void addEdge(int source, int target) throws NodeNotFoundException {
	if (! containsNode(source)) {
	    throw new NodeNotFoundException(source);
	} else if (! containsNode(target)) {
	    throw new NodeNotFoundException(target);
	} else {
	    matrix[source][target] = 1;
	} 
    }

    public void removeEdge(int source, int target) {
	if (containsNode(source) && containsNode(target)) {
	    matrix[source][target] = 0;
	}
    }

    public void removeNode(int node) {
	if(containsNode(node)) {
	    status[node] = FREE;
	    for (int j = 0; j < status.length; j++) {
		matrix[node][j] = 0;
		matrix[j][node] = 0;
	    }
	    cnt--;
	}
    }


    public Set<Integer> successors(int node) {
	Set<Integer> s = new TreeSet();
	if(containsNode(node)) {
	    for (int i = 0; i < status.length; i++) {
		if (containsEdge(node, i)) {
		    s.add(i);
		}
	    }
	}
	return s;
    }

    public Set<Integer> predecessors(int node) {
	Set<Integer> s = new TreeSet();
	if(containsNode(node)) {
	    for (int i = 0; i < status.length; i++) {
		if (containsEdge(i, node)) {
		    s.add(i);
		}
	    }
	}
	return s;
    }

    public int inDegree(int node) {
	return predecessors(node).size();
    }

    public int outDegree(int node) {
	return successors(node).size();
    }

}
